/******************************************************************************
	TalkBot_Slave4.c      v1.0    29th July 2009 - Roman Black
	Open-source-firmware for TalkBotBrain product from BlackRobotics.com
	This firmware is open-source for use by legitimate owners
	who purchased a TalkBotBrain. All rights to the TalkBot hardware
	circuitry design remain property of BlackRobotics.

	This is TalkBot_Slave4 - It is a simple sound player,
	it does not drive servos. It plays 1 of 4 sounds, and is
	controlled by 4 buttons on PORTA and PORTB. It does not
	use RX or TX serial commands. It still does serial download
	of sound library from PC at 19200 baud.
	
	This is basically the same as Slave3 which uses 8 buttons
	but this Slave only uses 4 buttons and does not require
	any pull-up resistors.

	TalkBot1 uses a PIC 16F628 and 2x 24LC512 i2c eeproms (1Megabit).

	Slave 4; The 4 buttons are on;
	 PORTB 4-7 (using PIC PORTB internal pull-ups)
	Buttons are active low, they connect to ground to play the sound.
	The sounds to be played must be sounds 0-3.

	For more TalkBot applications and FREE Sound Library files;
	 www.TalkBotBrain.com

// NOTE! Set TABS to 4
******************************************************************************/
/*
_BODEN_OFF   // these configs are set in MikroC compiler options.
_PWRTE_ON
_WDT_ON
_LVP_OFF
_MCLRE_OFF
_HS_OSC
*/
// vars in general operation, main() and int() etc;

#define PIN_LED1 PORTB.F3	// hi=ON, shared with BTc sound out
#define PIN_LED2 PORTB.F2	// lo=ON, shared with USART TX out, disabled if USART on

#define PIN_BUT1 PORTB.F1	// lo=pressed, shared with USART RX in
#define PIN_BUT2 PORTA.F5	// lo=pressed, general use button

unsigned char t0_toggle;	// for period timing with TMR0
unsigned char t0_count;		// period timing, debouncing

//unsigned char RXbyte1;		// the 2-byte command received by RX serial
//unsigned char RXbyte2;		// (not used in this version Slave)

unsigned char whichsound;	// which sound to play when button2 pressed
//unsigned int temp16;		// used in main() for any 16bit math

// used for Slave4 only;
unsigned char buttons1;		// holds 4 buttons for debouncing
unsigned char buttons2;		//


//---------------------------------------------------------
// (Servo8 system is not used in this version Slave)

//---------------------------------------------------------
// used in RI2C.C for my i2c eeprom functions

unsigned char ri2c_i absolute 0x20;	// MUST be in register bank0
unsigned char data;

unsigned char ri2c_add_hi;		// 16bit address within external i2c eeprom
unsigned char ri2c_add_lo;		//
unsigned char ri2c_add_chip;	//

#define	RBTC_PIN_SDA	PORTA.F4	// SDA pin, i2c eeprom
#define	RBTC_TRIS_SDA	TRISA.F4	// SDA pin, i2c eeprom
#define	RBTC_PIN_SCL	PORTB.F0	// SCL pin, i2c eeprom
#define	RBTC_PIN_BTC	PORTB.F3	// sound output pin

#define NO_ACK	0				// for i2c to control ack bit
#define ACK		1

// functions in RI2C.c

void ri2c_start_bit(void);
void ri2c_stop_bit(void);
void ri2c_send_byte(void);
void ri2c_receive_byte(unsigned char);

//---------------------------------------------------------
// vars in RBTC.c for BTc 1bit sound playback

#define	RBTC_SDA	PORTA,4			// the same 4 pin defines, syntax ASM only
#define	RBTC_TSDA	TRISA,4			//
#define	RBTC_SCL	PORTB,0			//
#define	RBTC_BTC	PORTB,3			//

// functions in RBTC.c
void play_btc_lib_sound(unsigned char thesound);

//---------------------------------------------------------
// vars in RSTREAM.c needed for serial data stream to eeprom

unsigned char timeout;			// timeout when stream is finished
unsigned char pos_in;			// pointer to position in input buffer
unsigned char input_full;		// flag to show we have 16 more input bytes

unsigned char in_buffer[32];	// buffer to hold incoming serial data stream
unsigned char out_buffer[32] absolute 0xB0;	// data being sent out to I2C eeprom

// functions in RSTREAM.c
void poll_serial_in(void);
void receive_stream(void);
void send_stream(void);
void rdelay_ms(unsigned char);	// used for mS delays anywhere

//-----------------------------------------------------------------------------


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  interrupt(void)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
{
	//----------------------------------------------------
	// (interrupt is not used in this version Slave)
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


//=============================================================================
#include "RI2C.c"		// my i2c eeprom functions
//=============================================================================
#include "RSTREAM.c"	// functions to do serial port data streaming
//=============================================================================
#include "RBTC.c"		// functions to do BTc 1bit sound playback
//=============================================================================


//=============================================================================
//  MAIN
//=============================================================================
void main()
{
	//---------------------------------------------
	// setup the PIC port registers etc

	CMCON = 0x07;			// set all pins to digital, comparators off

	PORTA =  0b00000000;	// RA4 must be kept low
	TRISA  = 0b00110000;  	// RA0-3 are 4 unused outputs
							// RA4 is SDA to eeprom, high imped
							// RA5 is but2 in

	PORTB =  0b00000101;	// make RB2 TX out high not-active
	TRISB  = 0b11110010;   	// RB0 is SCL out, RB1 is RX in
							// RB2 is TX out, RB3 is sound out
	                        // RB4-7 are 4 input buttons

	//-------------------------------------------------
 	// TIMER setups (PIC is 20MHz xtal)

	// TMR0 is used for general timing; mS delays, debouncing
	OPTION_REG = 0b00000111;	// TMR0 ON, at 256:1, PORTB pullups ON

	// TMR1 is used in interrupt for Servo8 system
	T1CON = 0b00000001;		// TMR1 ON, at 1:1 (5MHz)

	// TMR2 is used in BTc sound playback to control bitrate
	T2CON = 0b00000100;		// TMR2 ON, at 1:1 (5MHz)

   	//-------------------------------------------------
	// now TMR0 is operating, make a small delay to let PIC pins settle
	rdelay_ms(100);

	// and make a LED flash (and distinctive sound beep) to show
	// everything powered up ok. we can re-use vars t0_count
	// t0_toggle and whichsound here as they are not needed yet.

	whichsound = 3;			// 3 beeps
	while(whichsound)
	{
		t0_count = 25;  	// of 25 LED pulses
		t0_toggle = 6;		// starting at 6mS period
		while(t0_count)
		{
			// pulse the LED (and sound! is on same pin)
			rdelay_ms(t0_toggle);   // LED off period
			PIN_LED1 = 1;
			rdelay_ms(1);			// LED on period
			PIN_LED1 = 0;

			if(t0_toggle >1) t0_toggle--;
			t0_count--;
		}
		whichsound--;
	}

   	//-------------------------------------------------
	// (serial RX/TX commands not used in this version Slave)

   	//-------------------------------------------------
	// (TMR1 interrupt is not used in this version Slave)
	INTCON = 0;		// all ints OFF

   	//-------------------------------------------------
	// setup any variables
	
	whichsound = 0;		// sound0 is first to play if button2 pressed

	t0_count = 0;		// clear button2 debounce counter

   	//-------------------------------------------------
	// TalkBot main run loop here
	while(1)
	{

		//-------------------------------------------------
		// check the 4 external buttons on PORTB 4-7
		// low = pressed

		// read the 4 external buttons on PORTB
		buttons1 = (PORTB & 0b11110000);

		if(buttons1 != 0b11110000)		// if any buttons are pressed
		{
			// wait 5mS for debouncing
			rdelay_ms(5);

			// read the 4 buttons AGAIN
			buttons2 = (PORTB & 0b11110000);

		    // if buttons still match we will play a sound
		    if(buttons1 == buttons2)
		    {
		        // ok! play a sound 0-3, priority goes to lowest sound
				whichsound = 0;

				while(buttons1.F4)	// loop and find first sound
				{
					buttons1 = (buttons1 >> 1);
					whichsound++;
				}
				
				// actually play the sound
				play_btc_lib_sound(whichsound);
		    }
		}


		//-------------------------------------------------
		// if BUT2 is held down for >3 seconds,
		// go into receive serial stream mode
		if(!PIN_BUT2)		// if but2 pressed
		{
			if(TMR0.F7 != t0_toggle.F0)   // if detected a 152Hz toggle
			{
				t0_count++;

				if(t0_count >= 166)		// if but2 pressed >3 seconds
				{
					t0_count = (152-20-1);	// make it flash LED straight away

					// go into serial receive mode here.
					// first turn USART on at 19200 baud!
					RCSTA = 0;
					TXSTA = 0b00100100;		// high BRGH mode
					SPBRG = 64;				// at 20Mhz gives 19231 baud
					RCSTA = 0b10010000;		// SPEN bit7 =1 enables serial port

					// flash LED1 and wait for first serial byte to arrive
					while(1)
					{
						asm clrwdt;
						// flash LED1 once per second; 152Hz/152
						if(TMR0.F7 != t0_toggle.F0)
                    	{
							t0_toggle++;
							t0_count++;
							if(t0_count == (152-20)) PIN_LED1 = 1; // LED1 ON
							if(t0_count >= 152)
							{
								t0_count = 0;
								PIN_LED1 = 0;	// LED1 OFF
							}
						}
						// check for first serial byte received
						if(PIR1.RCIF == 1)
						{
							receive_stream();
							t0_count = 0;
							whichsound = 0;		// ready to play first sound
						    break;
						}
					}

					// receive is done, turn USART off again...
					RCSTA = 0;
				}
			}
		}
		else    // else BUT2 is released
		{
			// if BUT2 was quick pressed, play a sound!
			if(t0_count > 4)	// 4 gives small BUT2 debounce
			{
				play_btc_lib_sound(whichsound);		// play a sound
				whichsound++;		// select next sound (full 0-255 range)
			}
			t0_count = 0;	// reset BUT2 debounce count
		}
	   	//-------------------------------------------------
		// if BUT1 pressed, reset back to sound0
		if(!PIN_BUT1)		// if but1 pressed
		{
			whichsound = 0;
		}
	   	//-------------------------------------------------
		// To reduce power consumption, put the PIC in sleep
		// mode. The watchdog will wake it up after 18mS.

		asm sleep;

	   	//-------------------------------------------------
	}
}
//-----------------------------------------------------------------------------






